/**
 * Context constructor.
 * @memberof module:apeman-commons-invocating/lib
 * @inner
 * @constructor InvocationContext
 * @param {object} values - InvocationContext values.
 */

"use strict";

/** @lends module:apeman-commons-invocating/lib~InvocationContext */
function InvocationContext() {
    var s = this;
    s.init.apply(s, arguments);
}

InvocationContext.prototype = {
    /** @constructs InvocationContext */
    init: function (values) {
        var s = this;
        s.set(values);
    },
    /**
     * Clone this worker.
     * @returns {InvocationContext} - Clone.
     */
    clone: function () {
        var s = this;
        var Constructor = s.constructor;
        return new Constructor(s.get());
    },
    /**
     * Set context values.
     * @param {object} data - Values to set.
     * @returns {InvocationContext}
     */
    set: function (data) {
        var s = this;
        Object.keys(data || {}).forEach(function (key) {
            s[key] = data[key];
        });
        return s;
    },
    /**
     * Get context values.
     * @returns {*}
     */
    get: function () {
        var s = this;
        var data = {};
        Object.keys(s).forEach(function (key) {
            data[key] = s[key];
        });
        return data;
    },
    constructor: InvocationContext
};

/**
 * Define a context.
 */
InvocationContext.define = function (values) {
    function Defined() {
        var s = this;
        s.init.apply(s, arguments);
    }

    Defined.prototype = new InvocationContext(values);
    Defined.prototype.constructor = Defined;

    return Defined;
};

module.exports = InvocationContext;