/**
 * Invocation context.
 * @memberof module:apeman-commons-invocating/lib
 * @inner
 * @constructor InvocationPerformer
 * @param {object} config - Performing configuration.
 * @param {object} config.context - Invocation context.
 */

var async = require('async'),
    InvocationLoader = require('./invocation_loader'),
    InvocationContext = require('./invocation_context');

/** @lends InvocationPerformer */
function InvocationPerformer() {
    var s = this;
    s.init.apply(s, arguments);
}

InvocationPerformer.prototype = {
    /** @constructs InvocationPerformer */
    init: function (config) {
        var s = this;
        config = config || {};
        s.context = new InvocationContext(config.context || {});
    },
    /**
     * Execute an invocation.
     * @param {object} invocation
     * @param {function} callback - Callback when done.
     */
    perform: function (invocation, callback) {
        var s = this;
        var loader = new InvocationLoader({});
        async.waterfall([
            function (callback) {
                loader.load(invocation.$invoke, callback);
            },
            function ($invoke, callback) {
                var $args = s._getAmbiguously(invocation, '$args|$arg|$argument|$arguments|args|arg|argument|arguments') || [],
                    $options = s._getAmbiguously(invocation, '$options|$option|$opt|$opts|options|option|opt|opts') || {};
                var context = s.context.clone().set({
                    $args: $args,
                    $options: $options
                });
                var args = [context].concat($args || []).concat($options || {}).concat(callback);
                $invoke.apply(context, args);
            }
        ], callback);
    },
    _getAmbiguously: function (data, ambiguousKey) {
        var keys = ambiguousKey.split(/\|/g);
        for (var i = 0, len = keys.length; i < len; i++) {
            var key = keys[i];
            var hit = data.hasOwnProperty(key) || (!!data[key]) || (data[key] === 0);
            if (hit) {
                return data[key];
            }
        }
        return null;
    }
};

module.exports = InvocationPerformer;
