/**
 * Test case for invocationLoader.
 * Runs with nodeunit.
 */

var InvocationLoader = require('../lib/invocation_loader.js');

exports['Resolve modules.'] = function (test) {
    var loader = new InvocationLoader();
    test.equal(loader.resolve('__not_existing__'), null);
    test.equal(loader.resolve(__filename), __filename);
    test.done();
};


exports['Load module.'] = function (test) {
    var loader = new InvocationLoader({
        basedir: require.resolve('../docs/examples/module-loader-example.js')
    });
    loader.load('argx', function (err, found) {
        test.ifError(err);
        test.ok(!!found);
        test.done();
    });
};

exports['Load invalid module.'] = function (test) {
    var loader = new InvocationLoader();
    loader.load('__Invalid_module__', function (err) {
        test.ok(!!err);
        test.done();
    });
};


exports['Define a loader.'] = function (test) {
    var CustomLoader = InvocationLoader.define({
        foo: 'bar'
    });
    var loader = new CustomLoader();
    test.equal(loader.foo, 'bar');
    loader.load('__Invalid_module__', function (err) {
        test.ok(!!err);
        test.done();
    });
};
